lowp vec4 overlay(lowp vec4 bottom, lowp vec4 top)
{
    if (bottom.a > 0.0)
        bottom.rgb = min(vec3(1.0), bottom.rgb / bottom.a);
    else
        return vec4(0.0);
    
    lowp vec4 color = vec4(top.a);
    
    color.r *= (bottom.r < 0.5) ? (2.0 * bottom.r * top.r) : (2.0 * (bottom.r + (1.0 - bottom.r) * top.r) - 1.0);
    color.g *= (bottom.g < 0.5) ? (2.0 * bottom.g * top.g) : (2.0 * (bottom.g + (1.0 - bottom.g) * top.g) - 1.0);
    color.b *= (bottom.b < 0.5) ? (2.0 * bottom.b * top.b) : (2.0 * (bottom.b + (1.0 - bottom.b) * top.b) - 1.0);
    
    color += (1.0 - color.a) * bottom;
    
    return vec4(bottom.a * color.rgb, bottom.a);
}

lowp vec4 hardlight(lowp vec4 bottom, lowp vec4 top)
{
    if (bottom.a > 0.0)
        bottom.rgb = min(vec3(1.0), bottom.rgb / bottom.a);
    else
        return vec4(0.0);
    
    lowp vec4 color = vec4(top.a);
    
    color.r *= (top.r < 0.5) ? (2.0 * bottom.r * top.r) : (2.0 * (bottom.r + (1.0 - bottom.r) * top.r) - 1.0);
    color.g *= (top.g < 0.5) ? (2.0 * bottom.g * top.g) : (2.0 * (bottom.g + (1.0 - bottom.g) * top.g) - 1.0);
    color.b *= (top.b < 0.5) ? (2.0 * bottom.b * top.b) : (2.0 * (bottom.b + (1.0 - bottom.b) * top.b) - 1.0);
    
    color += (1.0 - color.a) * bottom;
    
    return vec4(bottom.a * color.rgb, bottom.a);
}

lowp vec4 softlightLegacy(lowp vec4 bottom, lowp vec4 top)
{
    if (bottom.a > 0.0)
        bottom.rgb = min(vec3(1.0), bottom.rgb / bottom.a);
    
    lowp vec4 color = vec4(top.a);
    
    color.r *= (2.0 * bottom.r * top.r) + (1.0 - 2.0 * bottom.r) * top.r * top.r;
    color.g *= (2.0 * bottom.g * top.g) + (1.0 - 2.0 * bottom.g) * top.g * top.g;
    color.b *= (2.0 * bottom.b * top.b) + (1.0 - 2.0 * bottom.b) * top.b * top.b;
    
    color += (1.0 - color.a) * bottom;
    
    return vec4(bottom.a * color.rgb, bottom.a);
}

lowp vec4 softlight(lowp vec4 bottom, lowp vec4 top)
{
    if (bottom.a > 0.0)
        bottom.rgb = min(vec3(1.0), bottom.rgb / bottom.a);
    
    lowp vec4 color = vec4(top.a);
    
    color.r *= ((top.r < 0.5) ? (2.0 * bottom.r * top.r + bottom.r * bottom.r * (1.0 - 2.0 * top.r)) : (sqrt(bottom.r) * (2.0 * top.r - 1.0) + 2.0 * bottom.r * (1.0 - top.r)));
    color.g *= ((top.g < 0.5) ? (2.0 * bottom.g * top.g + bottom.g * bottom.g * (1.0 - 2.0 * top.g)) : (sqrt(bottom.g) * (2.0 * top.g - 1.0) + 2.0 * bottom.g * (1.0 - top.g)));
    color.b *= ((top.b < 0.5) ? (2.0 * bottom.b * top.b + bottom.b * bottom.b * (1.0 - 2.0 * top.b)) : (sqrt(bottom.b) * (2.0 * top.b - 1.0) + 2.0 * bottom.b * (1.0 - top.b)));
    
    color += (1.0 - color.a) * bottom;
    
    return vec4(bottom.a * color.rgb, bottom.a);
}

lowp vec4 multiply(lowp vec4 bottom, lowp vec4 top)
{
    if (bottom.a > 0.0)
        bottom.rgb = min(vec3(1.0), bottom.rgb / bottom.a);
    
    lowp vec4 color = vec4(top.a);
    
    color.rgb *= bottom.rgb * top.rgb;
    color += (1.0 - color.a) * bottom;
    
    return vec4(bottom.a * color.rgb, bottom.a);
}

#define GrainMode_Alpha 0
#define GrainMode_Overlay 1
#define GrainMode_Hardlight 2
#define GrainMode_Softlight 3
#define GrainMode_SoftlightLegacy 4
#define GrainMode_Multiply 5

lowp vec4 grainBlendMode(int mode, lowp vec4 src, lowp float grain, lowp float factor, lowp float invGamma)
{
    grain = pow(grain, invGamma);
    if (mode == GrainMode_SoftlightLegacy)
        src = softlightLegacy(src, vec4(vec3(grain), factor));
    else if (mode == GrainMode_Softlight)
        src = softlight(src, vec4(vec3(grain), factor));
    else if (mode == GrainMode_Hardlight)
        src = hardlight(src, vec4(vec3(grain), factor));
    else if (mode == GrainMode_Overlay)
        src = overlay(src, vec4(vec3(grain), factor));
    else if (mode == GrainMode_Multiply)
        src = multiply(src, vec4(vec3(grain), factor));
    else // GrainMode_Alpha
        src = (1.0 - factor * grain) * src;
    
    return src;
}

lowp float colorburnf(lowp float bottom, lowp float top)
{
    if(top > 0.0)
        return max(1.0 - (1.0 - bottom) / top, 0.0);
    else
        return top;
}

lowp float overlayf(lowp float bottom, lowp float top)
{
    if(bottom < 0.5)
        return 2.0 * bottom * top;
    else
        return 1.0 - 2.0 * (1.0 - bottom) * (1.0 - top);
}

lowp float colordodgef(lowp float bottom, lowp float top)
{
    return (top < 1.0) ? min(bottom / (1.0 - top), 1.0) : top;
}

lowp float substractf(lowp float bottom, lowp float top)
{
    return max(bottom + top - 1.0, 0.0);
}

lowp float darkenf(lowp float blend, lowp float target)
{
    return min(blend, target);
}

lowp float multiplyf(lowp float blend, lowp float target)
{
    return blend * target;
}

lowp float linearburnf(lowp float blend, lowp float target)
{
    return target + blend - 1.0;
}

lowp float lightenf(lowp float blend, lowp float target)
{
    return max(blend, target);
}

lowp float screenf(lowp float blend, lowp float target)
{
    return 1.0 - (1.0 - blend) * (1.0 - target);
}

lowp float lineardodgef(lowp float blend, lowp float target)
{
    return target + blend;
}

lowp float softlightf(lowp float blend, lowp float target)
{
    if (blend > 0.5)
        return screenf(blend - 0.5, target);
    
    return multiplyf(blend + 0.5, target);
}

lowp float hardlightf(lowp float blend, lowp float target)
{
    return overlayf(target, blend);
}

lowp float vividlightf(lowp float bottom, lowp float top)
{
    return (top < 0.5) ? colorburnf(bottom, 2.0 * top) : colordodgef(bottom, 2.0 * (top - 0.5));
}

lowp float linearlightf(lowp float blend, lowp float target)
{
    if (blend > 0.5)
        return lineardodgef(2.0 * (blend - 0.5), target);
    
    return linearburnf(2.0 * blend, target);
}

lowp float pinlightf(lowp float blend, lowp float target)
{
    if (blend > 0.5)
        return lightenf(2.0 * (blend - 0.5), target);
    
    return darkenf(2.0 * blend, target);
}

lowp float differencef(lowp float blend, lowp float target)
{
    return abs(target - blend);
}

lowp float exclusionf(lowp float blend, lowp float target)
{
    return 0.5 - 2.0 * (target - 0.5) * (blend - 0.5);
}

lowp float hardmixf(lowp float bottom, lowp float top)
{
    return (bottom < 0.5 || top < 0.5) ? 0.0 : 1.0;
}

lowp float hardmix2f(lowp float bottom, lowp float top)
{
    return vividlightf(bottom, top) < 0.5? 0.0 : 1.0;
}

lowp float alphaWet(lowp float bottom, lowp float top, lowp float wetOpacity)
{
    return bottom * mix(1.0, wetOpacity, top);
}

lowp float alphaWetPL(lowp float bottom, lowp float top, lowp float wetOpacity)
{
    lowp float c = top < 0.5 ? top * 2.0 : mix(1.0, wetOpacity, (top - 0.5) * 2.0);
    return bottom * c;
}

lowp float levelRange(lowp float value, lowp float minInput, lowp float maxInput) {
    return min(max(value - minInput, 0.0) / (maxInput - minInput), 1.0);
}

#define AlphaMode_None 0
#define AlphaMode_NoneHardMix 1
#define AlphaMode_Multiply 2
#define AlphaMode_HardMix 3
#define AlphaMode_Overlay 4
#define AlphaMode_ColorBurn 5
#define AlphaMode_VividLight 6
#define AlphaMode_Wet 7
#define AlphaMode_WetInverse 8
#define AlphaMode_WetPL 9
#define AlphaMode_BlendMultiply 10
#define AlphaMode_BlendMultiplyAlphaLocked 11

lowp vec4 alphaBlending(int mode, lowp float modeLevel, lowp vec4 alpha, lowp vec4 src, lowp vec4 color, lowp float wetOpacity)
{
    if ((mode == AlphaMode_BlendMultiply) || (mode == AlphaMode_BlendMultiplyAlphaLocked))
    {
        lowp float dstAlpha = alpha.a;
        
        if (mode == AlphaMode_BlendMultiplyAlphaLocked) {
            if (dstAlpha > 0.0)
            {
                alpha = vec4((1.0 / dstAlpha) * alpha.rgb, 1.0);
            }
            else
                return vec4(0.0);
        } else {
            dstAlpha = 1.0;
        }
        
        return dstAlpha * (src * (alpha + vec4(1.0 - dstAlpha)) + vec4(1.0 - src.a) * alpha) * color;
    }
    
    lowp float srcAlpha = src.a;
    
    // DEMULTIPLY
    if (srcAlpha > 0.0)
        src = vec4((1.0 / srcAlpha) * src.rgb, 1.0);
    else
        src = vec4(0.0);
    
    lowp float r = 0.0;
    if(mode == AlphaMode_VividLight)
    {
        r = vividlightf(srcAlpha, alpha.a);
    }
    else if (mode == AlphaMode_ColorBurn)
    {
        r = colorburnf(srcAlpha, alpha.a);
    }
    else if (mode == AlphaMode_Overlay)
    {
        r = overlayf(srcAlpha, alpha.a);
    }
    else if(mode == AlphaMode_HardMix)
    {
        r = hardmixf(srcAlpha, alpha.a);
    }
    else if(mode == AlphaMode_Wet)
    {
        r = alphaWet(srcAlpha, alpha.a, wetOpacity);
    }
    else if(mode == AlphaMode_WetInverse)
    {
        r = alphaWet(alpha.a, srcAlpha, wetOpacity);
    }
    else if(mode == AlphaMode_WetPL)
    {
        r = alphaWetPL(srcAlpha, alpha.a, wetOpacity);
    }
    else // AlphaMode_Multiply
    {
        r = srcAlpha * alpha.a;
    }
    
    lowp float levelThreshold = min(0.5 - 0.01, 0.5 * modeLevel);
    r = levelRange(r, levelThreshold, 1.0 - levelThreshold);
    
    src = r * src * color;
     
    return src;
}

lowp vec4 applyAlphaModeLevel(lowp vec4 col, lowp float alphaModeLevel) {
    lowp float alpha = col.a;
    if(alpha > 0.0) {
        lowp float levelThreshold = min(0.5 - 0.01, 0.5 * alphaModeLevel);
        col.a = levelRange(alpha, levelThreshold, 1.0 - levelThreshold);
        col.rgb *= col.a / alpha;
    }
    return col;
}
