#include "blendmode.glsl"
#include "layers-blendmodes.glsl"

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3; // auxiliary layer
uniform sampler2D inputImageTexture4; // alpha auxiliary layer


uniform lowp vec4 color;
uniform lowp vec4 color1;
uniform lowp vec4 color2;

uniform int compositingMode;
uniform int alphaMode;
uniform ivec3 grainModes;
uniform bvec3 grainFormats;
uniform bool alphaLocked;
uniform bool wet;

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate1;


void main()
{
    lowp vec4 src = texture2D(inputImageTexture, textureCoordinate);
#ifdef auxiliaryLayerBlendMode
    lowp vec4 aux = texture2D(inputImageTexture3, textureCoordinate1);
    src = blendLayer(src, aux, auxiliaryLayerBlendMode, false);
#endif

    lowp vec4 dst = texture2D(inputImageTexture1, textureCoordinate);
    
    lowp vec4 alpha = texture2D(inputImageTexture2, textureCoordinate);
#ifdef auxiliaryAlphaLayerBlendMode
    lowp vec4 alphaAux = texture2D(inputImageTexture4, textureCoordinate1);
    alpha = blendLayer(alpha, alphaAux, auxiliaryAlphaLayerBlendMode, false);
#endif

    
    lowp float alphaModeLevel = color2.g;
    lowp float alphaModeWetOpacity = color2.a;
    
    if(!(wet && alphaMode == AlphaMode_BlendMultiplyAlphaLocked)) {
        src = alphaBlending(alphaMode, alphaModeLevel, alpha, src, color, alphaModeWetOpacity);
    }
    
    if(wet && alphaMode != AlphaMode_BlendMultiplyAlphaLocked) {
        dst = alphaBlending(alphaMode, alphaModeLevel, alpha, dst, vec4(1.0), alphaModeWetOpacity);
    }
    
    gl_FragColor = blendLayer(src, dst, layerBlendMode, alphaLocked);// * color1;
    
    if(wet && alphaMode == AlphaMode_BlendMultiplyAlphaLocked) {
        gl_FragColor = alphaBlending(AlphaMode_BlendMultiplyAlphaLocked, 0.0, alpha, gl_FragColor, vec4(1.0), 0.0);
    }

    gl_FragColor *= color1;
}
