#include "blendmode.glsl"

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;

// color[0] = undefined
// color[1] = blendMode
// color[2] = textureFormat
// color[3] = inverted
uniform mediump vec4 color;

// color1[0] = undefined
// color1[1] = textureGamma
// color1[2] = textureHeightFactor
// color1[3] = textureModeFactor
uniform lowp vec4 color1;

// color2[0] = level clip
// color2[1] = level gamma
uniform lowp vec4 color2;

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate1;
varying lowp vec4 vertexColor;
varying lowp float textureBlendFactor;

lowp float blend(lowp float blend, lowp float target, lowp int mode)
{
    lowp float result = 0.0;

    if (mode == 0) result = darkenf(blend, target);
    else if (mode == 1) result = multiplyf(blend, target);
    else if (mode == 2) result = colorburnf(blend, target);
    else if (mode == 3) result = linearburnf(blend, target);
    else if (mode == 4) result = lightenf(blend, target);
    else if (mode == 5) result = screenf(blend, target);
    else if (mode == 6) result = colordodgef(blend, target);
    else if (mode == 7) result = lineardodgef(blend, target);
    else if (mode == 8) result = overlayf(blend, target);
    else if (mode == 9) result = softlightf(blend, target);
    else if (mode == 10) result = hardlightf(blend, target);
    else if (mode == 11) result = vividlightf(blend, target);
    else if (mode == 12) result = linearlightf(blend, target);
    else if (mode == 13) result = pinlightf(blend, target);
    else if (mode == 14) result = differencef(blend, target);
    else if (mode == 15) result = exclusionf(blend, target);
    else result = hardmix2f(blend, target);

    return result;
}

lowp float height(lowp  float blend, lowp float target, lowp float depth)
{
    lowp float result = 0.0;
    if (depth < 0.5)
        result = darkenf(2.0 * depth * blend, target);
    else
        result = darkenf((2.0 * depth - 1.0) * (1.0 - blend) + blend, target);
    return result;
}

lowp float heightPLMultiply(lowp float blend, lowp float target, lowp float depth)
{
    blend = mix(0.0, blend, min(depth * 2.0, 1.0));
    lowp float extra = mix(0.0, target, max((depth - 0.5) * 2.0, 0.0));
    //blend = mix(blend, 1.0, max((depth - 0.5) * 2.0, 0.0));
    
    return min(target * blend + extra, 1.0);
}

lowp float heightPLMultiplyLinear(lowp float blend, lowp float target, lowp float depth)
{
    blend = blend * depth;
    lowp float extra = target * depth;
    
    return min(target * blend + extra, 1.0);
}

lowp float heightPLSubstract(lowp float blend, lowp float target, lowp float depth)
{
     blend = mix(0.0, blend, min(depth * 2.0, 1.0));
     blend = mix(blend, 1.0, max((depth - 0.5) * 2.0, 0.0));
    
     return substractf(target, blend);
}

lowp float heightPLSubstractKrita(lowp float blend, lowp float target, lowp float depth)
{
    lowp float pressureOffset = 1.0 - depth;
    return max(target - (blend + pressureOffset), 0.0);
}

void main()
{
    lowp float brush = texture2D(inputImageTexture, textureCoordinate).a;
    highp int mode = int(color[1]);

    bool grayscale = (int(color[2]) == 1);
    bool inverted = (int(color[3]) == 1);
    
    lowp float texture = (grayscale ? texture2D(inputImageTexture1, textureCoordinate1).r : texture2D(inputImageTexture1, textureCoordinate1).a);
    
    texture = inverted ? texture : (1.0 - texture);
               
    // gamma correction
    texture = pow(texture, color1.g);
    
    lowp float alpha = 0.0;
    
    if (mode == 17)
        alpha = height(texture, brush, textureBlendFactor);
    else if(mode == 18)
        alpha = heightPLMultiply(texture, brush, textureBlendFactor);
    else if(mode == 19)
        alpha = heightPLMultiplyLinear(texture, brush, textureBlendFactor);
    else if(mode == 20)
        alpha = heightPLSubstract(texture, brush, textureBlendFactor);
    else if(mode == 21)
        alpha = heightPLSubstractKrita(texture, brush, textureBlendFactor);
    else {
        alpha = blend(texture, brush, mode) * textureBlendFactor;
    }
    
    alpha = min(alpha + brush * color1.b * textureBlendFactor, 1.0);

    // contrast
    alpha = pow(alpha, color2.g);
    lowp float modeLevel = color2.r;
    lowp float levelThreshold = min(0.5 - 0.01, 0.5 * modeLevel);
    alpha = levelRange(alpha, levelThreshold, 1.0 - levelThreshold);
    
    // texture mode factor
    alpha = mix(brush, alpha, color1.a);
    
    gl_FragColor = vertexColor * vec4(vec3(1.0), alpha);
}
