uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3; // mask texture

uniform lowp vec4 color;
uniform lowp vec4 color1;
uniform lowp vec4 color2;
uniform lowp vec4 color3; // mask color

varying highp vec2 textureCoordinate;

lowp int mask_error(lowp int maskR, lowp int maskG, lowp int maskB, lowp int colorR, lowp int colorG, lowp int colorB)
{
    lowp int max = 0;
    lowp int error = 0;
    
    if (maskR > colorR)
        max = maskR - colorR;
    else
        max = colorR - maskR;
    
    if (maskG > colorG)
        error = maskG - colorG;
    else
        error = colorG - maskG;
    
    if (error > max)
        max = error;
    
    if (maskB > colorB)
        error = maskB - colorB;
    else
        error = colorB - maskB;
    
    if (error > max)
        return error;
    
    return max;
}

void main()
{
    lowp vec4 dst = texture2D(inputImageTexture1, textureCoordinate);
    lowp int useMask = (color3.a > -0.5) ? 1 : 0;
    lowp vec4 mask = (useMask == 1) ? texture2D(inputImageTexture3, textureCoordinate) : vec4(0.0, 0.0, 0.0, 1.0);
    
    lowp int maskA = int(floor(255.0 * mask.a + 0.5));
    
    if ( maskA < 5 )
    {
        gl_FragColor = dst;
    }
    else
    {
        lowp int maskR = int(floor(255.0 * mask.r + 0.5));
        lowp int maskG = int(floor(255.0 * mask.g + 0.5));
        lowp int maskB = int(floor(255.0 * mask.b + 0.5));
        
        lowp int colorR = int(floor(255.0 * color3.r + 0.5));
        lowp int colorG = int(floor(255.0 * color3.g + 0.5));
        lowp int colorB = int(floor(255.0 * color3.b + 0.5));
        
        if ( mask_error(maskR, maskG, maskB, colorR, colorG, colorB) < 5 )
        {
            lowp vec4 src = texture2D(inputImageTexture, textureCoordinate);
            
            lowp float alpha = texture2D(inputImageTexture2, textureCoordinate).r;
            
            if (color2.r > 0.5)
            {
                lowp float srcAlpha = src.a;
                
                if (srcAlpha > 0.0)
                    src = vec4((1.0 / srcAlpha) * src.rgb, 1.0);
                else
                    src = vec4(0.0);
                
                if (color2.r > 2.5)
                {
                    if ( (alpha > 0.0) && (((1.0 - srcAlpha) / alpha) < 1.0) )
                        src = (1.0 - ((1.0 - srcAlpha) / alpha)) * src;
                    else
                        src = vec4(0.0);
                }
                else if (color2.r > 1.5)
                {
                    if (srcAlpha < 0.5)
                        src = 2.0 * alpha * srcAlpha * src * color;
                    else
                        src = (1.0 - 2.0 * (1.0 - alpha) * (1.0 - srcAlpha)) * src * color;
                }
                else
                {
                    if ( (alpha < 0.5) || (srcAlpha < 0.5) )
                        src = vec4(0.0);
                    else
                        src = src * color;
                }
            }
            else
            {
                src = alpha * src * color;
            }
            
            if (color2.a > 0.0)
            {
                if (color2.r > 0.5)
                {
                    lowp float dstAlpha = dst.a;
                    
                    if (dstAlpha > 0.0)
                        dst = vec4((1.0 / dstAlpha) * dst.rgb, 1.0);
                    else
                        dst = vec4(0.0);
                    
                    if (color2.r > 2.5)
                    {
                        if ( (alpha > 0.0) && (((1.0 - dstAlpha) / alpha) < 1.0) )
                            dst = (1.0 - ((1.0 - dstAlpha) / alpha)) * dst;
                        else
                            dst = vec4(0.0);
                    }
                    else if (color2.r > 1.5)
                    {
                        if (dstAlpha < 0.5)
                            dst = 2.0 * alpha * dstAlpha * dst;
                        else
                            dst = (1.0 - 2.0 * (1.0 - alpha) * (1.0 - dstAlpha)) * dst;
                    }
                    else
                    {
                        if ( (alpha < 0.5) || (dstAlpha < 0.5) )
                            dst = vec4(0.0);
                    }
                }
                else
                {
                    dst = alpha * dst;
                }
            }
            
            gl_FragColor = (src * (dst + vec4((1.0 - dst.a))) + vec4(1.0 - src.a) * dst) * color1;
        }
        else
        {
            gl_FragColor = dst;
        }
    }
}
