uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;
uniform sampler2D inputImageTexture2; // mask texture
uniform sampler2D inputImageTexture3; // auxiliary layer

uniform  lowp vec4 color;
uniform  lowp vec4 color1;
uniform  lowp vec4 color2; // mask color
uniform  lowp vec4 color3;

varying  highp vec2 textureCoordinate;
varying  highp vec2 textureCoordinate1;

void alphaLockedDraw(lowp vec4 src, lowp vec4 dst)
{
    lowp float alpha = dst.a;
    if (alpha > 0.0)
    {
        dst = vec4((1.0 / alpha) * dst.rgb, 1.0);
        src = ((1.0 - src.a) * dst) * color1;
        gl_FragColor = alpha * src;
    }
    else
    {
        gl_FragColor = vec4(0.0);
    }
}

void draw(lowp vec4 src, lowp vec4 dst)
{
    gl_FragColor = ((1.0 - src.a) * dst) * color1;
}

lowp int mask_error(lowp int maskR, lowp int maskG, lowp int maskB, lowp int colorR, lowp int colorG, lowp int colorB)
{
    lowp int max = 0;
    lowp int error = 0;
    
    if (maskR > colorR)
        max = maskR - colorR;
    else
        max = colorR - maskR;
    
    if (maskG > colorG)
        error = maskG - colorG;
    else
        error = colorG - maskG;
    
    if (error > max)
        max = error;
    
    if (maskB > colorB)
        error = maskB - colorB;
    else
        error = colorB - maskB;
    
    if (error > max)
        return error;
    
    return max;
}

void main()
{
    lowp vec4 dst = texture2D(inputImageTexture1, textureCoordinate);
    lowp int useMask = (color2.a > -0.5) ? 1 : 0;
    lowp vec4 mask = (useMask == 1) ? texture2D(inputImageTexture2, textureCoordinate) : vec4(0.0, 0.0, 0.0, 1.0);
    
    lowp int maskA = int(floor(255.0 * mask.a + 0.5));
    
    if ( maskA < 5 )
    {
        gl_FragColor = dst;
    }
    else
    {
        lowp int maskR = int(floor(255.0 * mask.r + 0.5));
        lowp int maskG = int(floor(255.0 * mask.g + 0.5));
        lowp int maskB = int(floor(255.0 * mask.b + 0.5));
        
        lowp int colorR = int(floor(255.0 * color2.r + 0.5));
        lowp int colorG = int(floor(255.0 * color2.g + 0.5));
        lowp int colorB = int(floor(255.0 * color2.b + 0.5));
        
        if ( mask_error(maskR, maskG, maskB, colorR, colorG, colorB) < 5 )
        {
            lowp vec4 src = texture2D(inputImageTexture, textureCoordinate) * color;
#ifdef auxiliaryLayerBlendMode
        lowp vec4 aux = texture2D(inputImageTexture3, textureCoordinate1);
        src = blendLayer(src, aux, auxiliaryLayerBlendMode, false) * color;
#endif
            if (color3.a > 0.5)
                alphaLockedDraw(src, dst);
            else
                draw(src, dst);
        }
        else
        {
            gl_FragColor = dst;
        }
    }
}
