uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform sampler2D inputImageTexture4;

uniform  lowp vec4 color;
uniform  lowp vec4 color1;
uniform  lowp vec4 color2;
uniform  lowp vec4 color3;

uniform  lowp vec4 color5; // alpha locked

varying  highp vec2 textureCoordinate;
varying  highp vec2 textureCoordinate1;
varying  highp vec2 textureCoordinate2;
varying  highp vec2 textureCoordinate3;

 lowp vec4 overlay( lowp vec4 bottom,  lowp vec4 top)
{
    if (bottom.a > 0.0)
        bottom.rgb = min(vec3(1.0), bottom.rgb / bottom.a);
    
     lowp vec4 color = vec4(top.a);
    
    color.r *= (bottom.r < 0.5) ? (2.0 * bottom.r * top.r) : (2.0 * (bottom.r + (1.0 - bottom.r) * top.r) - 1.0);
    color.g *= (bottom.g < 0.5) ? (2.0 * bottom.g * top.g) : (2.0 * (bottom.g + (1.0 - bottom.g) * top.g) - 1.0);
    color.b *= (bottom.b < 0.5) ? (2.0 * bottom.b * top.b) : (2.0 * (bottom.b + (1.0 - bottom.b) * top.b) - 1.0);
    
    color += (1.0 - color.a) * bottom;
    
    return vec4(bottom.a * color.rgb, bottom.a);
}

 lowp vec4 hardlight(lowp vec4 bottom, lowp vec4 top)
{
    if (bottom.a > 0.0)
        bottom.rgb = min(vec3(1.0), bottom.rgb / bottom.a);
    
     lowp vec4 color = vec4(top.a);
    
    color.r *= (top.r < 0.5) ? (2.0 * bottom.r * top.r) : (2.0 * (bottom.r + (1.0 - bottom.r) * top.r) - 1.0);
    color.g *= (top.g < 0.5) ? (2.0 * bottom.g * top.g) : (2.0 * (bottom.g + (1.0 - bottom.g) * top.g) - 1.0);
    color.b *= (top.b < 0.5) ? (2.0 * bottom.b * top.b) : (2.0 * (bottom.b + (1.0 - bottom.b) * top.b) - 1.0);
    
    color += (1.0 - color.a) * bottom;
    
    return vec4(bottom.a * color.rgb, bottom.a);
}

void main()
{
    lowp vec4 src = texture2D(inputImageTexture, textureCoordinate);
    lowp vec4 dst = texture2D(inputImageTexture1, textureCoordinate);
    
    lowp float alphaMode = color2.a / 10.0;
    if (alphaMode > 0.5)
    {
        lowp float srcAlpha = src.a;
        
        if (srcAlpha > 0.0)
            src = vec4((1.0 / srcAlpha) * src.rgb, 1.0);
        else
            src = vec4(0.0);
        
        if (srcAlpha < 0.5)
            src = vec4(0.0);
        else
            src = src * color;
    }
    else
    {
        src = src * color;
    }
    
    lowp vec4 grainFormat = color3 / 10.0;
    lowp vec4 grainMode = mod(color3, 10.0);
    
    if (color2.r > 0.0)
    {
        lowp vec4 grainRGBA = texture2D(inputImageTexture2, textureCoordinate1);
        lowp float grain = grainFormat.r > 0.5 ? grainRGBA.r : grainRGBA.a;
        
        if (grainMode.r > 1.5)
            src = hardlight(src, vec4(vec3(grain), color2.r));
        else if (grainMode.r > 0.5)
            src = overlay(src, vec4(vec3(grain), color2.r));
        else
            src = (1.0 - color2.r * grain) * src;
    }
    
    if (color2.g > 0.0)
    {
        lowp vec4 grainRGBA = texture2D(inputImageTexture3, textureCoordinate2);
        lowp float grain = grainFormat.g > 0.5 ? grainRGBA.r : grainRGBA.a;
        
        if (grainMode.g > 1.5)
            src = hardlight(src, vec4(vec3(grain), color2.g));
        else if (grainMode.g > 0.5)
            src = overlay(src, vec4(vec3(grain), color2.g));
        else
            src = (1.0 - color2.g * grain) * src;
    }
    
    if (color2.b > 0.0)
    {
        lowp vec4 grainRGBA = texture2D(inputImageTexture4, textureCoordinate3);
        lowp float grain = grainFormat.b > 0.5 ? grainRGBA.r : grainRGBA.a;
        
        if (grainMode.b > 1.5)
            src = hardlight(src, vec4(vec3(grain), color2.b));
        else if (grainMode.b > 0.5)
            src = overlay(src, vec4(vec3(grain), color2.b));
        else
            src = (1.0 - color2.b * grain) * src;
    }
    
    if(color5.a > 0.5)
    {
        // alpha locked
        lowp float alpha = dst.a;
        if (alpha > 0.0)
        {
            dst = vec4((1.0 / alpha) * dst.rgb, 1.0);
            src = min(src, dst) * color1;
            gl_FragColor = alpha * src;
        }
        else
        {
            gl_FragColor = vec4(0.0);
        }
    }
    else
    {
        gl_FragColor = min(src, dst) * color1;
    }
}
