uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform sampler2D inputImageTexture4;
uniform sampler2D inputImageTexture5; // mask texture

uniform  lowp vec4 color;
uniform  lowp vec4 color1;
uniform  lowp vec4 color2;
uniform  lowp vec4 color3;
uniform  lowp vec4 color4; // mask color
uniform  lowp vec4 color5; // alpha locked

varying  highp vec2 textureCoordinate;
varying  highp vec2 textureCoordinate1;
varying  highp vec2 textureCoordinate2;
varying  highp vec2 textureCoordinate3;

lowp vec4 overlay( lowp vec4 bottom,  lowp vec4 top)
{
    if (bottom.a > 0.0)
        bottom.rgb = min(vec3(1.0), bottom.rgb / bottom.a);
    
    lowp vec4 color = vec4(top.a);
    
    color.r *= (bottom.r < 0.5) ? (2.0 * bottom.r * top.r) : (2.0 * (bottom.r + (1.0 - bottom.r) * top.r) - 1.0);
    color.g *= (bottom.g < 0.5) ? (2.0 * bottom.g * top.g) : (2.0 * (bottom.g + (1.0 - bottom.g) * top.g) - 1.0);
    color.b *= (bottom.b < 0.5) ? (2.0 * bottom.b * top.b) : (2.0 * (bottom.b + (1.0 - bottom.b) * top.b) - 1.0);
    
    color += (1.0 - color.a) * bottom;
    
    return vec4(bottom.a * color.rgb, bottom.a);
}

lowp vec4 hardlight(lowp vec4 bottom, lowp vec4 top)
{
    if (bottom.a > 0.0)
        bottom.rgb = min(vec3(1.0), bottom.rgb / bottom.a);
    
    lowp vec4 color = vec4(top.a);
    
    color.r *= (top.r < 0.5) ? (2.0 * bottom.r * top.r) : (2.0 * (bottom.r + (1.0 - bottom.r) * top.r) - 1.0);
    color.g *= (top.g < 0.5) ? (2.0 * bottom.g * top.g) : (2.0 * (bottom.g + (1.0 - bottom.g) * top.g) - 1.0);
    color.b *= (top.b < 0.5) ? (2.0 * bottom.b * top.b) : (2.0 * (bottom.b + (1.0 - bottom.b) * top.b) - 1.0);
    
    color += (1.0 - color.a) * bottom;
    
    return vec4(bottom.a * color.rgb, bottom.a);
}


lowp int mask_error(lowp int maskR, lowp int maskG, lowp int maskB, lowp int colorR, lowp int colorG, lowp int colorB)
{
    lowp int max = 0;
    lowp int error = 0;
    
    if (maskR > colorR)
        max = maskR - colorR;
    else
        max = colorR - maskR;
    
    if (maskG > colorG)
        error = maskG - colorG;
    else
        error = colorG - maskG;
    
    if (error > max)
        max = error;
    
    if (maskB > colorB)
        error = maskB - colorB;
    else
        error = colorB - maskB;
    
    if (error > max)
        return error;
    
    return max;
}

void main()
{
    lowp vec4 dst = texture2D(inputImageTexture1, textureCoordinate);
    lowp int useMask = (color4.a > -0.5) ? 1 : 0;
    lowp vec4 mask = (useMask == 1) ? texture2D(inputImageTexture5, textureCoordinate) : vec4(0.0, 0.0, 0.0, 1.0);
    
    lowp int maskA = int(floor(255.0 * mask.a + 0.5));
    
    if ( maskA < 5 )
    {
        gl_FragColor = dst;
    }
    else
    {
        lowp int maskR = int(floor(255.0 * mask.r + 0.5));
        lowp int maskG = int(floor(255.0 * mask.g + 0.5));
        lowp int maskB = int(floor(255.0 * mask.b + 0.5));
        
        lowp int colorR = int(floor(255.0 * color4.r + 0.5));
        lowp int colorG = int(floor(255.0 * color4.g + 0.5));
        lowp int colorB = int(floor(255.0 * color4.b + 0.5));
        
        if ( mask_error(maskR, maskG, maskB, colorR, colorG, colorB) < 5 )
        {
            lowp vec4 src = texture2D(inputImageTexture, textureCoordinate);
            
            lowp float alphaMode = color2.a / 10.0;
            if (alphaMode > 0.5)
            {
                lowp float srcAlpha = src.a;
                
                if (srcAlpha > 0.0)
                    src = vec4((1.0 / srcAlpha) * src.rgb, 1.0);
                else
                    src = vec4(0.0);
                
                if (srcAlpha < 0.5)
                    src = vec4(0.0);
                else
                    src = src * color;
            }
            else
            {
                src = src * color;
            }
            
            lowp vec4 grainFormat = color3 / 10.0;
            lowp vec4 grainMode = mod(color3, 10.0);
            
            if (color2.r > 0.0)
            {
                lowp vec4 grainRGBA = texture2D(inputImageTexture2, textureCoordinate1);
                lowp float grain = grainFormat.r > 0.5 ? grainRGBA.r : grainRGBA.a;
                
                if (grainMode.r > 1.5)
                    src = hardlight(src, vec4(vec3(grain), color2.r));
                else if (grainMode.r > 0.5)
                    src = overlay(src, vec4(vec3(grain), color2.r));
                else
                    src = (1.0 - color2.r * grain) * src;
            }
            
            if (color2.g > 0.0)
            {
                lowp vec4 grainRGBA = texture2D(inputImageTexture3, textureCoordinate2);
                lowp float grain = grainFormat.g > 0.5 ? grainRGBA.r : grainRGBA.a;
                
                if (grainMode.g > 1.5)
                    src = hardlight(src, vec4(vec3(grain), color2.g));
                else if (grainMode.g > 0.5)
                    src = overlay(src, vec4(vec3(grain), color2.g));
                else
                    src = (1.0 - color2.g * grain) * src;
            }
            
            if (color2.b > 0.0)
            {
                lowp vec4 grainRGBA = texture2D(inputImageTexture4, textureCoordinate3);
                lowp float grain = grainFormat.b > 0.5 ? grainRGBA.r : grainRGBA.a;
                
                if (grainMode.b > 1.5)
                    src = hardlight(src, vec4(vec3(grain), color2.b));
                else if (grainMode.b > 0.5)
                    src = overlay(src, vec4(vec3(grain), color2.b));
                else
                    src = (1.0 - color2.b * grain) * src;
            }
                        
            if (color5.a > 0.5)
            {
                lowp float alpha = dst.a;
                if (alpha > 0.0)
                {
                    dst = vec4((1.0 / alpha) * dst.rgb, 1.0);
                    src = (src + (vec4(1) - src) * dst) * color1;
                    gl_FragColor = alpha * src;
                }
                else
                {
                    gl_FragColor = vec4(0.0);
                }
            }
            else
            {
                gl_FragColor = (src + (vec4(1) - src) * dst) * color1;
            }
        }
        else
        {
            gl_FragColor = dst;
        }
    }
}
