#include "blendmode.glsl"
#include "layers-blendmodes.glsl"

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform sampler2D inputImageTexture4;
uniform sampler2D inputImageTexture5; // mask texture
uniform sampler2D inputImageTexture6; // auxiliary layer

uniform  lowp vec4 color;
uniform  lowp vec4 color1;
uniform  lowp vec4 color2; // grain factor
uniform  lowp vec4 color3; // grain gamma
uniform  lowp vec4 color4; // mask color

uniform int compositingMode;
uniform int alphaMode;
uniform ivec3 grainModes;
uniform bvec3 grainFormats;
uniform bool alphaLocked;
uniform bool wet;

varying  highp vec2 textureCoordinate;
varying  highp vec2 textureCoordinate1;
varying  highp vec2 textureCoordinate2;
varying  highp vec2 textureCoordinate3;
varying  highp vec2 textureCoordinate4;

lowp int mask_error(lowp int maskR, lowp int maskG, lowp int maskB, lowp int colorR, lowp int colorG, lowp int colorB)
{
    lowp int max = 0;
    lowp int error = 0;
    
    if (maskR > colorR)
        max = maskR - colorR;
    else
        max = colorR - maskR;
    
    if (maskG > colorG)
        error = maskG - colorG;
    else
        error = colorG - maskG;
    
    if (error > max)
        max = error;
    
    if (maskB > colorB)
        error = maskB - colorB;
    else
        error = colorB - maskB;
    
    if (error > max)
        return error;
    
    return max;
}

void main()
{
    lowp vec4 dst = texture2D(inputImageTexture1, textureCoordinate);
    lowp int useMask = (color4.a > -0.5) ? 1 : 0;
    lowp vec4 mask = (useMask == 1) ? texture2D(inputImageTexture5, textureCoordinate) : vec4(0.0, 0.0, 0.0, 1.0);
    
    lowp int maskA = int(floor(255.0 * mask.a + 0.5));
    
    if ( maskA < 5 )
    {
        gl_FragColor = dst;
    }
    else
    {
        lowp int maskR = int(floor(255.0 * mask.r + 0.5));
        lowp int maskG = int(floor(255.0 * mask.g + 0.5));
        lowp int maskB = int(floor(255.0 * mask.b + 0.5));
        
        lowp int colorR = int(floor(255.0 * color4.r + 0.5));
        lowp int colorG = int(floor(255.0 * color4.g + 0.5));
        lowp int colorB = int(floor(255.0 * color4.b + 0.5));
        
        if ( mask_error(maskR, maskG, maskB, colorR, colorG, colorB) < 5 )
        {
            lowp vec4 src = texture2D(inputImageTexture, textureCoordinate);
#ifdef auxiliaryLayerBlendMode
            lowp vec4 aux = texture2D(inputImageTexture6, textureCoordinate4);
            src = blendLayer(src, aux, auxiliaryLayerBlendMode, false);
#endif
            
            src = applyAlphaModeLevel(src, color3.a);
            
            if(alphaMode == AlphaMode_NoneHardMix) {
                if (src.a > 0.5)
                    src = vec4((1.0 / src.a) * src.rgb, 1.0) * color;
                else
                    src = vec4(0.0);
            }
            else {
                src = src * color;
            }
            
            if (color2.r > 0.0)
            {
                lowp vec4 grainRGBA = texture2D(inputImageTexture2, textureCoordinate1);
                lowp float grain = grainFormats.r ? grainRGBA.r : grainRGBA.a;
                
                src = grainBlendMode(grainModes.r, src, grain, color2.r, color3.r);
            }
            
            if (color2.g > 0.0)
            {
                lowp vec4 grainRGBA = texture2D(inputImageTexture3, textureCoordinate2);
                lowp float grain = grainFormats.g ? grainRGBA.r : grainRGBA.a;
                
                src = grainBlendMode(grainModes.g, src, grain, color2.g, color3.g);
            }
            
            if (color2.b > 0.0)
            {
                lowp vec4 grainRGBA = texture2D(inputImageTexture4, textureCoordinate3);
                lowp float grain = grainFormats.b ? grainRGBA.r : grainRGBA.a;
                
                src = grainBlendMode(grainModes.b, src, grain, color2.b, color3.b);
            }
            
            gl_FragColor = blendLayer(src, dst, layerBlendMode, alphaLocked) * color1;
        }
        else
        {
            gl_FragColor = dst;
        }
    }
}
