#include "blendmode.glsl"
#include "layers-blendmodes.glsl"

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;
uniform sampler2D inputImageTexture2; // mask texture
uniform sampler2D inputImageTexture3; // auxiliary layer

uniform  lowp vec4 color;
uniform  lowp vec4 color1;
uniform  lowp vec4 color2;
uniform  lowp vec4 color3; // mask color

uniform int compositingMode;
uniform int alphaMode;
uniform ivec3 grainModes;
uniform bvec3 grainFormats;
uniform bool alphaLocked;
uniform bool wet;

varying  highp vec2 textureCoordinate;
varying  highp vec2 textureCoordinate1;

lowp int mask_error(lowp int maskR, lowp int maskG, lowp int maskB, lowp int colorR, lowp int colorG, lowp int colorB)
{
    lowp int max = 0;
    lowp int error = 0;
    
    if (maskR > colorR)
        max = maskR - colorR;
    else
        max = colorR - maskR;
    
    if (maskG > colorG)
        error = maskG - colorG;
    else
        error = colorG - maskG;
    
    if (error > max)
        max = error;
    
    if (maskB > colorB)
        error = maskB - colorB;
    else
        error = colorB - maskB;
    
    if (error > max)
        return error;
    
    return max;
}

void main()
{
    lowp vec4 dst = texture2D(inputImageTexture1, textureCoordinate);
    lowp int useMask = (color3.a > -0.5) ? 1 : 0;
    lowp vec4 mask = (useMask == 1) ? texture2D(inputImageTexture2, textureCoordinate) : vec4(0.0, 0.0, 0.0, 1.0);
    
    lowp int maskA = int(floor(255.0 * mask.a + 0.5));
    
    if ( maskA < 5 )
    {
        gl_FragColor = dst;
    }
    else
    {
        lowp int maskR = int(floor(255.0 * mask.r + 0.5));
        lowp int maskG = int(floor(255.0 * mask.g + 0.5));
        lowp int maskB = int(floor(255.0 * mask.b + 0.5));
        
        lowp int colorR = int(floor(255.0 * color3.r + 0.5));
        lowp int colorG = int(floor(255.0 * color3.g + 0.5));
        lowp int colorB = int(floor(255.0 * color3.b + 0.5));
        
        if ( mask_error(maskR, maskG, maskB, colorR, colorG, colorB) < 5 )
        {
            lowp vec4 src = texture2D(inputImageTexture, textureCoordinate) * color;
#ifdef auxiliaryLayerBlendMode
            lowp vec4 aux = texture2D(inputImageTexture3, textureCoordinate1);
            src = blendLayer(src, aux, auxiliaryLayerBlendMode, false);
#endif
            
            src = applyAlphaModeLevel(src, color2.g);
            
            gl_FragColor = blendLayer(src, dst, layerBlendMode, alphaLocked) * color1;
        }
        else
        {
            gl_FragColor = dst;
        }
    }
}
